package com.agentfactory.agentspeak.interpreter.mentalstate;

import com.agentfactory.agentspeak.interpreter.AbstractAgentSpeakAgent;
import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.Bindings;
import com.agentfactory.logic.update.reasoner.QueryResult;

public class BasicStep implements IPlanStep {
	private IFormula step;
	
	public BasicStep(IFormula step) {
		this.step = step;
	}
	
	@Override
	public IPlanStep apply(Bindings set) {
		return new BasicStep(step.apply(set));
	}
	
	public IFormula formula() {
		return step;
	}

	public String toString() {
		return Utilities.presenter.toString(step);
	}

	@Override
	public void handle(AbstractAgentSpeakAgent agent, Commitment commitment) {
		IFormula activity = step.apply(commitment.bindings());
		if (TestFormula.class.isInstance(activity)) {
			QueryResult result = agent.queryAll(((TestFormula) activity).formula());
			if (result.found) {
				commitment.updateBindings(result.bindings.get(0));
			}
		} else if (StateUpdateFormula.class.isInstance(activity)) {
			StateUpdateFormula formula = (StateUpdateFormula) activity;
			if (formula.updateType() == StateUpdateFormula.ADDITION) {
				if (Predicate.class.isInstance(formula.formula())) {
					Predicate predicate = (Predicate) formula.formula().apply(new Bindings());
					Predicate selfSource = (Predicate) Utilities.factory.createFormula("source(self)");
					predicate.getLabels().add(selfSource);
					agent.addBelief(predicate, null);
				}
			} else if (formula.updateType() == StateUpdateFormula.REMOVAL) {
				if (Predicate.class.isInstance(formula.formula())) {
					Predicate predicate = (Predicate) formula.formula().apply(new Bindings());
					Predicate selfSource = (Predicate) Utilities.factory.createFormula("source(self)");
					predicate.getLabels().add(selfSource);
					agent.removeBelief(predicate, null);
				}
			}
		} else if (AchievementFormula.class.isInstance(activity)) {
			agent.addEvent(new ASEvent(ASEvent.ADDITION, (Predicate) ((AchievementFormula) activity).formula(), commitment));
			commitment.suspend();
		} else if (Predicate.class.isInstance(activity)){
			if (!agent.performActivity((Predicate) activity)) {
				// NEED TO DEAL WITH SUCCESS OR FAILURE OF ACTIVITY...
				commitment.fail();
			}
		}
	}
	
}
