package com.agentfactory.agentspeak.interpreter.mentalstate;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.agentspeak.interpreter.AbstractAgentSpeakAgent;
import com.agentfactory.logic.update.reasoner.Bindings;

public class TryRecoverStep implements IPlanStep {
	private List<IPlanStep> plan;
	private List<IPlanStep> recoverPlan;
	
	public TryRecoverStep(List<IPlanStep> plan, List<IPlanStep> recoverPlan) {
		this.plan = plan;
		this.recoverPlan = recoverPlan;
	}
	@Override
	public IPlanStep apply(Bindings set) {
		List<IPlanStep> tryList = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			tryList.add(step.apply(set));
		}
		List<IPlanStep> recoverList = new LinkedList<IPlanStep>();
		for (IPlanStep step: recoverPlan) {
			recoverList.add(step.apply(set));
		}
		return new TryRecoverStep(tryList, recoverList);
	}

	public String toString() {
		String output = "try {";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		output += "} recover {";
		first = true;
		for (IPlanStep step: recoverPlan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		return output + "}";
	}

	public List<IPlanStep> plan() {
		return plan;
	}

	public List<IPlanStep> recoverPlan() {
		return recoverPlan;
	}
	@Override
	public void handle(AbstractAgentSpeakAgent agent, Commitment commitment) {
		List<IPlanStep> steps = new LinkedList<IPlanStep>();
		for (IPlanStep stp: recoverPlan) {
			steps.add(stp);
		}
		commitment.addRecoveryPlan(steps);
		
		steps = new LinkedList<IPlanStep>();
		for (IPlanStep stp: plan) {
			steps.add(stp);
		}
		steps.add(this);
		commitment.addPlan(steps);
	}
}
