package com.agentfactory.logic.agent.beliefManagement;

import com.agentfactory.logic.agent.*;
import com.agentfactory.logic.MentalAgent;
import com.agentfactory.logic.lang.FOS;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * A Perception-based belief manager supports the capture of perceptions via
 * the Agent Factory Perceptor component and their storage within multiple
 * belief stores (one per perceptor). All belief stores are wiped when the
 * update() method is invoked and new beliefs are extracted from each
 * perceptor.
 *
 * The implementation does not cater for explict adoption of beliefs (hence
 * it is declared abstract).
 *
 * @author rem
 */
public abstract class PerceptionBasedBeliefManager implements IBeliefManager {
    protected Map<String, BeliefStore> perceptions;
    protected MentalAgent agent;

    public PerceptionBasedBeliefManager(MentalAgent agent) {
        this.agent = agent;
        perceptions = new HashMap<String, BeliefStore>();
    }
    
    @Override
    public void update() {
        BeliefStore store = null;
        for (Perceptor perceptor: agent.getPerceptors()) {
            store = perceptions.get(perceptor.getClass().getName());
            if (store == null) {
                store = new BeliefStore(perceptor.getClass().getName());
                perceptions.put(store.getLabel(), store);
            }
            perceptor.fire();
            store.refresh(perceptor.getPerceptions());
        }
    }

    public List query(FOS query) {
        List<FOS> list = new LinkedList<FOS>();
        for (BeliefStore store: perceptions.values()) {
            list.addAll(store.query(query));
        }
        return list;
    }

}
