/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.platform.core;

import java.util.Collection;
import java.util.List;
import java.util.Observer;

/**
 *
 * @author remcollier
 */
public interface IAgent extends Observer {
    /**
     * FIPA INITIATED STATE
     */
    public static final int INITIATED                   = 0;
    
    /**
     * FIPA ACTIVE STATE
     */
    public static final int ACTIVE                      = 1;
    
    /**
     * FIPA WAITING STATE
     */
    public static final int WAITING                     = 2;
    
    /**
     * FIPA SUSPENDED STATE
     */
    public static final int SUSPENDED                   = 3;

    /**
     * FIPA TRANSIT STATE
     */
    public static final int TRANSIT                     = 4;

    /**
     * FIPA TERMINATED STATE
     */
    public static final int TERMINATED                  = 5;
    
    /**
     * SCHEDULER NOTIFICATION STRING
     */
    public static final String END_OF_TIME_SLICE     = "end-of-time-slice";
    
    /**
     * Get the name of the agent
     * 
     * @return a String representation of the agent's name
     */
    public String getName();
    
    public String getType();
    
    /**
     * Get the state of the agent
     * 
     * @return an integer value representing the state which is one of INITIATED,
     * ACTIVE, WAITING, SUSPENDED, TRANSIT, TERMINATED
     */
    public int getState();
    
    /**
     * Set the state of the agent
     * 
     * @param state an integer value representing the state which is one of INITIATED,
     * ACTIVE, WAITING, SUSPENDED, TRANSIT, TERMINATED
     */
    public void setState(int state);
    
    /**
     * 
     * @param container
     */
    public void setContainer(IAgentContainer container);
    
    /**
     * 
     * @param id
     * @return
     */
    public IPlatformService getPlatformService(String id);
    public boolean bindToPlatformService(String id);
    public Collection<IPlatformService> getBoundPlatformServices();
    /**
     * Initialise the agent with the data.
     * 
     * @param data
     */
    public void initialise(String data);
    
    /**
     * Scheduler Methods
     */
    public void execute();
    public void setTimeSlice(long timeSlice);
    public void setEndOfTimeSlice(boolean endOfTimeSlice);
    public long getTimeSlice();
    public long getIterations();

    // Communication Support
    public void receive(IMessage message);
    public boolean send(IMessage message);
    public AgentID getAgentID();
    public List<IMessage> getInbox();
    public List<IMessage> getOutbox();
}
