/**
 * Title:       Message.java
 * Copyright:   Copyright (c) 1996-2004 The Agent Factory Working Group. All rights reserved.
 * Licence:     This file is free software; you can redistribute it and/or modify
 *              it under the terms of the GNU Lesser General Public License as published by
 *              the Free Software Foundation; either version 2.1, or (at your option)
 *              any later version.
 *
 *              This file is distributed in the hope that it will be useful,
 *              but WITHOUT ANY WARRANTY; without even the implied warranty of
 *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *              GNU Lesser General Public License for more details.
 *
 *              You should have received a copy of the GNU Lesser General Public License
 *              along with Agent Factory; see the file COPYING.  If not, write to
 *              the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *              Boston, MA 02111-1307, USA.
 */
package com.agentfactory.platform.core;


import java.util.List;
/**
 * This class implements a standard FIPA ACL Message.  It should be sublclassed
 * for specific implementations of the standard (i.e. string encoding, xml
 * encoding).
 *
 * @author  Rem Collier
 */
public interface IMessage {
    // Performatives
    public static final String ACCEPT_PROPOSAL                  = "accept-proposal";
    public static final String ACTION                           = "action";
    public static final String AGREE                            = "agree";
    public static final String CANCEL                           = "cancel";
    public static final String CALL_FOR_PROPOSAL                = "cfp";
    public static final String CONFIRM                          = "confirm";
    public static final String DISCONFIRM                       = "disconfirm";
    public static final String FAILURE                          = "failure";
    public static final String INFORM                           = "inform";
    public static final String INFORM_IF                        = "inform-if";
    public static final String INFORM_REF                       = "inform-ref";
    public static final String NOT_UNDERSTOOD                   = "not-understood";
    public static final String PROPOGATE                        = "propogate";
    public static final String PROPOSE                          = "propose";
    public static final String PROXY                            = "proxy";
    public static final String QUERY_IF                         = "query-if";
    public static final String QUERY_REF                        = "query-ref";
    public static final String REFUSE                           = "refuse";
    public static final String REJECT_PROPOSAL                  = "reject_proposal";
    public static final String REQUEST                          = "request";
    public static final String REQUEST_WHEN                     = "request-when";
    public static final String REQUEST_WHENEVER                 = "request-whenever";
    public static final String SUBSCRIBE                        = "subscribe";
    
    public String getPerformative();
    public AgentID getSender();
    public List<AgentID> getReceivers();
    public String getContent();
    public AgentID getReplyTo();
    public String getReplyWith();
    public String getLanguage();
    public String getOntology();
    public int getContentLength();
    public String getAclRepresentation();
    public String getPayloadEncoding();
    public String getProtocol();
    public String getConversationId();
    public byte[] toByteArray();
}
    
    
