package com.agentfactory.platform.impl;

import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentContainer;
import com.agentfactory.platform.core.IAgentPlatform;
import com.agentfactory.platform.core.IArchitectureService;
import com.agentfactory.platform.core.IPlatformService;
import com.agentfactory.platform.core.IPlatformServiceManager;
import com.agentfactory.platform.core.IScheduler;
import com.agentfactory.platform.core.NoSuchServiceException;
import com.agentfactory.service.ams.AgentManagementService;
import com.agentfactory.service.mts.local.LocalMessageTransportService;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author remcollier
 * @author daithi
 */
public class DefaultAgentPlatform extends AbstractAgentContainer implements IAgentPlatform {
    private IArchitectureService architectureService;
    private IScheduler scheduler;
    private IPlatformServiceManager serviceManager;
    private String name;
    private String domain;
    
    public DefaultAgentPlatform() {
        super();

        this.scheduler = new RoundRobinTimeSliceScheduler();
        this.architectureService = new ArchitectureService();
        this.serviceManager = new PlatformServiceManager(this);
        
        try {
            ((PlatformServiceManager) serviceManager).addService(AgentManagementService.class, AgentManagementService.NAME);
            ((PlatformServiceManager) serviceManager).addService(LocalMessageTransportService.class, LocalMessageTransportService.NAME);
        } catch (NoSuchServiceException ex) {
            Logger.getLogger(DefaultAgentPlatform.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }

    public void add(IAgent agent) throws DuplicateAgentNameException {
        super.add(agent);
        
        // Bind to all available MTS
        for (IPlatformService mts: serviceManager.getFilteredServices(MessageTransportService.class)) {
            serviceManager.bindToService(agent, mts);
        }
    }
    
    public IAgentContainer getAgentContainer() {
        return this;
    }
    
    public IScheduler getScheduler() {
        return scheduler;
    }
    
    public IArchitectureService getArchitectureService() {
        return architectureService;
    }
    
    public IPlatformServiceManager getPlatformServiceManager() {
        return this.serviceManager;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public void setDomain(String domain) {
        this.domain = domain;
    }
    
    public String getFullName() {
        return name + "." + domain;
    }
    
    public String getName() {
        return name;
    }
    
    public String getDomain() {
        return domain;
    }

    public IPlatformService bindToPlatformService(IAgent agent, String id) {
        IPlatformService service = serviceManager.getServiceByName(id);
//        System.out.println("Service: " + service);
        serviceManager.bindToService(agent, service);
        return service;
    }

    public List<IPlatformService> bindToMessageTransportServices(IAgent agent) {
        List<IPlatformService> mts = serviceManager.getFilteredServices(MessageTransportService.class);
        for (IPlatformService service: mts) {
            serviceManager.bindToService(agent,service);
        }
        return mts;
    }
    
    public void setScheduler(IScheduler scheduler) {
        this.scheduler = scheduler;
    }
}
