/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.platform.impl;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IScheduler;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author remcollier
 */
public final class MultiThreadScheduler implements IScheduler {
    private static String LOGNAME = MultiThreadScheduler.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.WARNING);
    }
    
    private Map<String, IAgent> agentMap;
    private Map<String, AgentThread> threadMap;
    private boolean terminated;
    
    public MultiThreadScheduler() {
        threadMap = new HashMap<String, AgentThread>();
        agentMap = new HashMap<String, IAgent>();
        terminated = false;
    }
    
    /**
     * Registers the agent with the scheduler for execution.
     * 
     * @param agent the agent to be scheduled
     */
    public synchronized void register(IAgent agent) {
        AgentThread agentThread = new AgentThread(agent);
        Thread thread = new Thread(agentThread, agent.getName());
        threadMap.put(agent.getName(), agentThread);
        agentMap.put(agent.getName(), agent);
        thread.setPriority(5);
        thread.start();
    }

    /**
     * Removed the given agent from the schedule.
     * 
     * @param agent the agent to be un-scheduled
     */
    public synchronized void unregister(IAgent agent) {
        agentMap.remove(agent.getName());
        threadMap.remove(agent.getName());
        
        // Agent will be terminated at the start of the next execution...
        agent.setState(IAgent.TERMINATED);
    }

    /**
     * Resume the given agent
     * 
     * @param name
     */
    public void resume(String name) {
        IAgent agent = agentMap.get(name);
        agent.setState(IAgent.ACTIVE);
    }

    /**
     * Suspend the given agent
     * 
     * @param name
     */
    public void suspend(String name) {
        IAgent agent = agentMap.get(name);
        agent.setState(IAgent.SUSPENDED);
    }

    /**
     * Step the given agent. This method is synchronised so that the agent
     * cannot be executed multiple times (concurrently).
     * 
     * @param name
     */
    public void step(String name) {
        IAgent agent = agentMap.get(name);
        agent.setState(IAgent.SUSPENDED);
        agent.execute();
    }
    
    /**
     * Implementation of Agent Control Thread
     */
    private class AgentThread implements Runnable {
        IAgent agent;
        
        public AgentThread(IAgent agent) {
            this.agent = agent;
        }

        public void run() {
            do {
                if (agent.getState() == IAgent.ACTIVE) {
                    agent.execute();
                }
                
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ex) {
                    Logger.getLogger(MultiThreadScheduler.class.getName()).log(Level.SEVERE, null, ex);
                }
            } while (!isTerminated());
        }
        
        private synchronized boolean isTerminated() {
            return agent.getState() == IAgent.TERMINATED;
        }
    }
}
