package com.agentfactory.teleoreactive.interpreter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Properties;

import com.agentfactory.logic.update.interpreter.AbstractLanguageAgent;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.ITerm;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.IQueryEngine;
import com.agentfactory.logic.update.reasoner.QueryResult;
import com.agentfactory.logic.update.reasoner.ResolutionBasedQueryEngine;
import com.agentfactory.platform.core.AgentID;
import com.agentfactory.teleoreactive.interpreter.mentalstate.ModelTower;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRProgram;
import com.agentfactory.teleoreactive.interpreter.mentalstate.RuleBase;

public abstract class AbstractTeleoReactiveAgent extends AbstractLanguageAgent {
	protected ModelTower beliefs;
	protected IQueryEngine engine;
	protected RuleBase BCrules;
	private Map<String, TRProgram> programs;
	protected String activeProgram;
	protected List<String> programStack;
	protected List<String> actionTrace;

	public AbstractTeleoReactiveAgent(String name) {
		super(name);
		actionTrace = new LinkedList<String>();
		beliefs = new ModelTower();
		engine = new ResolutionBasedQueryEngine();
		BCrules = new RuleBase();
		engine.addSource(beliefs);
		engine.addSource(BCrules);
		activeProgram = "main:0";
		setPrograms(new HashMap<String, TRProgram>());
		programStack = new LinkedList<String>();
		actionTrace = new LinkedList<String>();
		
		installBasicAPI();
	}

	@Override
	public void execute() {
		this.retrieveNewMessages();
		senseEnvironment();
        beliefs.endOfPerception();
        this.getOutbox().clear();
        
		IFormula activity = getPrograms().get(activeProgram).execute(engine, new LinkedList<ITerm>());
		while (getPrograms().containsKey(Utilities.createSignature((Predicate) activity))) {
			programStack.add(Utilities.presenter.toString(activity));
			if (Predicate.class.isInstance(activity)) {
				Predicate potentialAction = ((Predicate) activity);
				LinkedList<ITerm> args = new LinkedList<ITerm>();
				for (int i = 0; i < potentialAction.size(); i++) {
					args.add(i, potentialAction.termAt(i));
				}
				activity = getPrograms().get(Utilities.createSignature((Predicate) activity)).execute(engine, args);
			}
		}
		if(actionTrace.isEmpty() || (!actionTrace.get(actionTrace.size()-1).equals(Utilities.presenter.toString(activity)) && !Utilities.presenter.toString(activity).equals(".nil")) )
			actionTrace.add(actionTrace.size(), Utilities.presenter.toString(activity));
		performActivity((Predicate) activity);
		endOfIteration();
	}

	public String getType() {
		return "AF-TeleoReactive";
	}

	/*public boolean performActivity(Predicate activity) {
		Action action = actionMap.get(Utilities.createSignature(activity));
		if (action == null) {
			if(activity.functor().contains(".")){
				String x[] = activity.functor().split("\\.");
				Module m = moduleMap.get(x[0]+":0");
				String sig = Utilities.createSignature(activity);
				String id = sig.substring(sig.indexOf(".")+1);
				Action a = m.getAction(id);
				if(a == null)
					return false;
				boolean s = a.execute(activity);
				if (s){
					actionTrace.add(actionTrace.size(), Utilities.presenter.toString(activity));
				}
				return s;

			}
		}
		System.out.println(Utilities.presenter.toString(activity));
		boolean s = action.execute(activity);
		if (s)
			actionTrace.add(actionTrace.size(), Utilities.presenter.toString(activity));
		return s;
	}*/

	public void initialise(String data) {
			IFormula act = Utilities.factory.createFormula(data);
			if (Predicate.class.isInstance(act))
				beliefs.addNewInformedBelief((Predicate) act);
		}

	@Override
	public void update(Observable o, Object arg) {}

	public List<String> getStack() {
		List<String> s = new LinkedList<String>();
		for (String string : programStack) {
			s.add(string);
		}

		programStack.clear();
		return s;
	}

	public List<String> getBeliefs() {
		return beliefs.getBeliefs();
	}

	public List<String> getActionTrace() {
		List<String> n = new LinkedList<String>();
		for (String s : actionTrace) {
			n.add(s);
		}
		return n;
	}
	
	public void addBelief(Predicate bel){
		beliefs.addBelief(bel);
	}

	public void addMessageBelief(AgentID sender, String content) {
		beliefs.addMesageBelief((Predicate) Utilities.factory.createFormula("comunicatingWith("+sender+")"));
		beliefs.addNewInformedBelief((Predicate) Utilities.factory.createFormula(content));
	}

	public void addSentBelief(AgentID agentID, String content) {
		beliefs.addMesageBelief((Predicate) Utilities.factory.createFormula("informed("+agentID+","+content+")"));
	}
    
	void processArgs(Properties arguments) {
        String argument = arguments.getProperty("TIMESLICE");
        System.out.println(argument);
        if (argument != null) {
            setTimeSlice(Integer.parseInt(argument));
        }
    }

	
	public boolean hasBelief(IFormula createFormula) {
		QueryResult result = engine.query(createFormula);
	    return result.found;
    }

	public void setPrograms(Map<String, TRProgram> programs) {
		this.programs = programs;
	}

	public Map<String, TRProgram> getPrograms() {
		return programs;
	}

}
