/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.visualiser.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentContainer;
import com.agentfactory.platform.core.NoSuchAgentException;
import com.agentfactory.visualiser.core.IStateManager;
import com.agentfactory.visualiser.core.IStateManagerFactory;

/**
 *
 * @author remcollier
 */
public class SnapshotManager extends Observable implements Observer {
    private static final String LOGNAME = SnapshotManager.class.getName();
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.WARNING);
    }
    
    private Map<String, IStateManagerFactory> managerMap;
    private Map<String, IStateManager> stateManagers;
    
    public SnapshotManager() {
        stateManagers = new HashMap<String, IStateManager>();
        managerMap = new HashMap<String, IStateManagerFactory>();
    }
    
    public void registerStateManagerFactory(IStateManagerFactory factory) {
		managerMap.put(factory.getType(), factory);
    }

    public void update(Observable source, Object event) {
        if (IAgentContainer.class.isInstance(source)) {
            IAgentContainer container = (IAgentContainer) source;

            Logger.getLogger(LOGNAME).log(Level.INFO, "Incoming Event: " + event);
            String[] details = event.toString().split(" ");
            if (details[0].equals("created")) {
                try {
                    IAgent agent = container.getAgentByName(details[1]);
                    
                    IStateManagerFactory factory = managerMap.get(agent.getType());
                    if (factory == null) {
                        factory = new DefaultStateManagerFactory();
                    }

                    IStateManager stateManager = factory.createStateManager(agent);
                    stateManagers.put(details[1], stateManager);
                    if (Observable.class.isInstance(agent)) {
                        ((Observable) agent).addObserver(this);
                    }
                } catch (NoSuchAgentException ex) {
                    Logger.getLogger(SnapshotManager.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            this.setChanged();
            notifyObservers(event);
        } else if (IAgent.class.isInstance(source)) {
            if (event.equals("end-of-time-slice")) {
                IAgent agent = (IAgent) source;
                IStateManager manager = stateManagers.get(agent.getName());
                manager.capture();
            }
        }
    }
    
    public IStateManager getStateManager(String name) {
        return stateManagers.get(name);
    }
}
